@@ -23,6 +23,7 @@
  */
   
 #include "cfg.h"
+#include "module-config.h"
 #include "cfg-tree.h"
 #include "messages.h"
 #include "template/templates.h"
@@ -185,6 +186,40 @@ cfg_set_mark_mode(GlobalConfig *self, gchar *mark_mode)
   self->mark_mode = cfg_lookup_mark_mode(mark_mode);
 }
 
+static void
+_invoke_module_init(gchar *key, ModuleConfig *mc, gpointer *args)
+{
+  GlobalConfig *cfg = (GlobalConfig *) args[0];
+  gboolean *result = (gboolean *) args[1];
+
+  if (!module_config_init(mc, cfg))
+    *result = FALSE;
+}
+
+static void
+_invoke_module_deinit(gchar *key, ModuleConfig *mc, gpointer user_data)
+{
+  GlobalConfig *cfg = (GlobalConfig *) user_data;
+
+  module_config_deinit(mc, cfg);
+}
+
+static gboolean
+cfg_init_modules(GlobalConfig *cfg)
+{
+  gboolean result = TRUE;
+  gpointer args[] = { cfg, &result };
+  g_hash_table_foreach(cfg->module_config, (GHFunc) _invoke_module_init, args);
+
+  return result;
+}
+
+static void
+cfg_deinit_modules(GlobalConfig *cfg)
+{
+  g_hash_table_foreach(cfg->module_config, (GHFunc) _invoke_module_deinit, cfg);
+}
+
 gboolean
 cfg_init(GlobalConfig *cfg)
 {
@@ -216,7 +251,7 @@ cfg_init(GlobalConfig *cfg)
         }
     }
 
-  if (!rcptid_init(cfg->state, cfg->use_rcptid))
+  if (!rcptid_init(cfg->state, cfg->use_uniqid))
     return FALSE;
 
   stats_reinit(&cfg->stats_options);
@@ -225,13 +260,15 @@ cfg_init(GlobalConfig *cfg)
   dns_cache_set_params(cfg->dns_cache_size, cfg->dns_cache_expire, cfg->dns_cache_expire_failed, cfg->dns_cache_hosts);
   hostname_reinit(cfg->custom_domain);
   host_resolve_options_init(&cfg->host_resolve_options, cfg);
-  log_proto_register_builtin_plugins(cfg);
+  if (!cfg_init_modules(cfg))
+    return FALSE;
   return cfg_tree_start(&cfg->tree);
 }
 
 gboolean
 cfg_deinit(GlobalConfig *cfg)
 {
+  cfg_deinit_modules(cfg);
   rcptid_deinit();
   return cfg_tree_stop(&cfg->tree);
 }
@@ -294,11 +331,18 @@ cfg_allow_config_dups(GlobalConfig *self)
     }
 }
 
+static void
+cfg_register_builtin_plugins(GlobalConfig *self)
+{
+  log_proto_register_builtin_plugins(self);
+}
+
 GlobalConfig *
 cfg_new(gint version)
 {
   GlobalConfig *self = g_new0(GlobalConfig, 1);
 
+  self->module_config = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, (GDestroyNotify) module_config_free);
   self->user_version = version;
 
   self->flush_lines = 100;
@@ -323,6 +367,7 @@ cfg_new(gint version)
   self->dns_cache_expire = 3600;
   self->dns_cache_expire_failed = 60;
   self->threaded = TRUE;
+  self->pass_unix_credentials = TRUE;
   
   log_template_options_defaults(&self->template_options);
   self->template_options.ts_format = TS_FMT_BSD;
@@ -337,10 +382,13 @@ cfg_new(gint version)
 
   self->recv_time_zone = NULL;
   self->keep_timestamp = TRUE;
+
+  self->use_uniqid = FALSE;
   
   stats_options_defaults(&self->stats_options);
 
   cfg_tree_init_instance(&self->tree, self);
+  cfg_register_builtin_plugins(self);
   return self;
 }
 
@@ -483,9 +531,10 @@ cfg_free(GlobalConfig *self)
   g_free(self->bad_hostname_re);
   g_free(self->dns_cache_hosts);
   g_free(self->custom_domain);
-  g_list_free(self->plugins);
+  plugin_free_plugins(self);
   plugin_free_candidate_modules(self);
   cfg_tree_free_instance(&self->tree);
+  g_hash_table_unref(self->module_config);
   g_free(self);
 }
 
@@ -555,3 +604,21 @@ cfg_persist_config_fetch(GlobalConfig *cfg, gchar *name)
   return res;
 }
 
+gint
+cfg_get_user_version(const GlobalConfig *cfg)
+{
+  return cfg->user_version;
+}
+
+gint
+cfg_get_parsed_version(const GlobalConfig *cfg)
+{
+  return cfg->parsed_version;
+}
+
+const gchar*
+cfg_get_filename(const GlobalConfig *cfg)
+{
+  return cfg->filename;
+}
+
