@@ -34,6 +34,8 @@
 #include "tls-support.h"
 #include "compat/string.h"
 #include "rcptid.h"
+#include "template/macros.h"
+#include "lib/host-id.h"
 
 #include <sys/types.h>
 #include <time.h>
@@ -155,7 +157,7 @@ TLS_BLOCK_END;
  **********************************************************************/
 
 static inline gboolean
-log_msg_chk_flag(LogMessage *self, gint32 flag)
+log_msg_chk_flag(const LogMessage *self, gint32 flag)
 {
   return self->flags & flag;
 }
@@ -172,6 +174,12 @@ log_msg_unset_flag(LogMessage *self, gint32 flag)
   self->flags &= ~flag;
 }
 
+static inline void
+log_msg_set_host_id(LogMessage *msg)
+{
+  msg->host_id = host_id_get();
+}
+
 /* the index matches the value id */
 const gchar *builtin_value_names[] =
 {
@@ -198,7 +206,7 @@ static StatsCounterItem *count_sdata_updates;
 static GStaticPrivate priv_macro_value = G_STATIC_PRIVATE_INIT;
 
 static inline gboolean
-log_msg_is_write_protected(LogMessage *self)
+log_msg_is_write_protected(const LogMessage *self)
 {
   return self->protect_cnt > 0;
 }
@@ -374,7 +382,7 @@ __free_macro_value(void *val)
 }
 
 const gchar *
-log_msg_get_macro_value(LogMessage *self, gint id, gssize *value_len)
+log_msg_get_macro_value(const LogMessage *self, gint id, gssize *value_len)
 {
   GString *value;
 
@@ -573,7 +581,11 @@ log_msg_set_value_indirect(LogMessage *self, NVHandle handle, NVHandle ref_handl
     log_msg_update_sdata(self, handle, name, name_len);
 }
 
-
+gboolean
+log_msg_values_foreach(LogMessage *self, NVTableForeachFunc func, gpointer user_data)
+{
+  return nv_table_foreach(self->payload, logmsg_registry, func, user_data);
+}
 
 void
 log_msg_set_match(LogMessage *self, gint index, const gchar *value, gssize value_len)
@@ -622,7 +634,7 @@ log_msg_clear_matches(LogMessage *self)
 #endif
 
 static inline void
-log_msg_tags_foreach_item(LogMessage *self, gint base, gulong item, LogMessageTagsForeachFunc callback, gpointer user_data)
+log_msg_tags_foreach_item(const LogMessage *self, gint base, gulong item, LogMessageTagsForeachFunc callback, gpointer user_data)
 {
   gint i;
 
@@ -642,7 +654,7 @@ log_msg_tags_foreach_item(LogMessage *self, gint base, gulong item, LogMessageTa
 
 
 void
-log_msg_tags_foreach(LogMessage *self, LogMessageTagsForeachFunc callback, gpointer user_data)
+log_msg_tags_foreach(const LogMessage *self, LogMessageTagsForeachFunc callback, gpointer user_data)
 {
   guint i;
 
@@ -800,7 +812,7 @@ log_msg_sdata_append_escaped(GString *result, const gchar *sstr, gssize len)
 }
 
 void
-log_msg_append_format_sdata(LogMessage *self, GString *result,  guint32 seq_num)
+log_msg_append_format_sdata(const LogMessage *self, GString *result,  guint32 seq_num)
 {
   const gchar *value;
   const gchar *sdata_name, *sdata_elem, *sdata_param, *cur_elem = NULL, *dot;
@@ -934,14 +946,14 @@ log_msg_append_format_sdata(LogMessage *self, GString *result,  guint32 seq_num)
 }
 
 void
-log_msg_format_sdata(LogMessage *self, GString *result,  guint32 seq_num)
+log_msg_format_sdata(const LogMessage *self, GString *result,  guint32 seq_num)
 {
   g_string_truncate(result, 0);
   log_msg_append_format_sdata(self, result, seq_num);
 }
 
 gboolean
-log_msg_append_tags_callback(LogMessage *self, LogTagId tag_id, const gchar *name, gpointer user_data)
+log_msg_append_tags_callback(const LogMessage *self, LogTagId tag_id, const gchar *name, gpointer user_data)
 {
   GString *result = (GString *) ((gpointer *) user_data)[0];
   gint original_length = GPOINTER_TO_UINT(((gpointer *) user_data)[1]);
@@ -956,7 +968,7 @@ log_msg_append_tags_callback(LogMessage *self, LogTagId tag_id, const gchar *nam
 }
 
 void
-log_msg_print_tags(LogMessage *self, GString *result)
+log_msg_print_tags(const LogMessage *self, GString *result)
 {
   gpointer args[] = { result, GUINT_TO_POINTER(result->len) };
 
@@ -994,6 +1006,7 @@ log_msg_init(LogMessage *self, GSockAddr *saddr)
   self->flags |= LF_STATE_OWN_MASK;
 
   self->rcptid = rcptid_generate_id();
+  log_msg_set_host_id(self);
 }
 
 void
@@ -1067,6 +1080,29 @@ log_msg_alloc(gsize payload_size)
   return msg;
 }
 
+static gboolean
+_merge_value(NVHandle handle, const gchar *name, const gchar *value, gssize value_len, gpointer user_data)
+{
+  LogMessage *msg = (LogMessage *) user_data;
+
+  if (!nv_table_is_value_set(msg->payload, handle))
+    log_msg_set_value(msg, handle, value, value_len);
+  return FALSE;
+}
+
+void
+log_msg_merge_context(LogMessage *self, LogMessage **context, gsize context_len)
+{
+  gint i;
+
+  for (i = context_len - 1; i >= 0; i--)
+    {
+      LogMessage *msg_to_be_merged = context[i];
+
+      log_msg_values_foreach(msg_to_be_merged, _merge_value, self);
+    }
+}
+
 /**
  * log_msg_new:
  * @msg: message to parse
@@ -1608,6 +1644,12 @@ log_msg_registry_deinit(void)
   logmsg_registry = NULL;
 }
 
+void
+log_msg_registry_foreach(GHFunc func, gpointer user_data)
+{
+  nv_registry_foreach(logmsg_registry, func, user_data);
+}
+
 void
 log_msg_global_init(void)
 {
@@ -1625,14 +1667,21 @@ log_msg_get_handle_name(NVHandle handle, gssize *length)
   return nv_registry_get_handle_name(logmsg_registry, handle, length);
 }
 
-gboolean
-log_msg_nv_table_foreach(NVTable *self, NVTableForeachFunc func, gpointer user_data)
-{
-  return nv_table_foreach(self, logmsg_registry, func, user_data);
-}
-
 void
 log_msg_global_deinit(void)
 {
   log_msg_registry_deinit();
 }
+
+const gchar *
+__log_msg_get_value(const LogMessage *self, NVHandle handle, gssize *value_len)
+__attribute__((alias("log_msg_get_value")));
+
+const gchar *
+__log_msg_get_value_by_name(const LogMessage *self, const gchar *name, gssize *value_len)
+__attribute__((alias("log_msg_get_value_by_name")));
+
+void
+__log_msg_set_value_by_name(LogMessage *self, const gchar *name, const gchar *value, gssize length)
+__attribute__((alias("log_msg_set_value_by_name")));
+
