@@ -28,7 +28,7 @@
 
 #include <iv.h>
 
-typedef enum { GENERAL_THREAD = 0, OUTPUT_THREAD, MAIN_LOOP_WORKER_TYPE_MAX} MainLoopWorkerType;
+typedef enum { GENERAL_THREAD = 0, OUTPUT_THREAD, EXTERNAL_INPUT_THREAD, MAIN_LOOP_WORKER_TYPE_MAX} MainLoopWorkerType;
 
 TLS_BLOCK_START
 {
@@ -77,15 +77,19 @@ _allocate_thread_id(void)
    * more threads than that, we can generalize this algorithm further. */
 
   main_loop_worker_id = 0;
-  for (id = 0; id < MAIN_LOOP_MAX_WORKER_THREADS; id++)
-    {
-      if ((main_loop_workers_idmap[main_loop_worker_type] & (1 << id)) == 0)
-        {
-          /* id not yet used */
 
-          main_loop_worker_id = (id + 1)  + (main_loop_worker_type * MAIN_LOOP_MAX_WORKER_THREADS);
-          main_loop_workers_idmap[main_loop_worker_type] |= (1 << id);
-          break;
+  if(main_loop_worker_type != EXTERNAL_INPUT_THREAD)
+    {
+      for (id = 0; id < MAIN_LOOP_MAX_WORKER_THREADS; id++)
+        {
+          if ((main_loop_workers_idmap[main_loop_worker_type] & (1 << id)) == 0)
+            {
+              /* id not yet used */
+
+              main_loop_worker_id = (id + 1)  + (main_loop_worker_type * MAIN_LOOP_MAX_WORKER_THREADS);
+              main_loop_workers_idmap[main_loop_worker_type] |= (1 << id);
+              break;
+            }
         }
     }
   g_static_mutex_unlock(&main_loop_workers_idmap_lock);
@@ -163,6 +167,10 @@ main_loop_worker_thread_start(void *cookie)
     {
       main_loop_worker_type = OUTPUT_THREAD;
     }
+  else if(worker_options && worker_options->is_external_input)
+    {
+      main_loop_worker_type = EXTERNAL_INPUT_THREAD;
+    }
 
   _allocate_thread_id();
   INIT_IV_LIST_HEAD(&batch_callbacks);
@@ -310,7 +318,7 @@ main_loop_create_worker_thread(WorkerThreadFunc func, WorkerExitNotificationFunc
   main_loop_worker_job_start();
   if (terminate_func)
     _register_exit_notification_callback(terminate_func, data);
-  h = g_thread_create_full(_worker_thread_func, p, 128 * 1024, FALSE, TRUE, G_THREAD_PRIORITY_NORMAL, NULL);
+  h = g_thread_create_full(_worker_thread_func, p, 1024 * 1024, FALSE, TRUE, G_THREAD_PRIORITY_NORMAL, NULL);
   g_assert(h != NULL);
 }
 
