@@ -25,6 +25,25 @@
 #include "msg-format.h"
 #include "cfg.h"
 #include "plugin.h"
+#include "plugin-types.h"
+
+void
+msg_format_inject_parse_error(LogMessage *msg, const guchar *data, gsize length)
+{
+  gchar buf[2048];
+
+  log_msg_clear(msg);
+
+  msg->timestamps[LM_TS_STAMP] = msg->timestamps[LM_TS_RECVD];
+  log_msg_set_value(msg, LM_V_HOST, "", 0);
+  g_snprintf(buf, sizeof(buf), "Error processing log message: %.*s", (gint) length, data);
+  log_msg_set_value(msg, LM_V_MESSAGE, buf, -1);
+  log_msg_set_value(msg, LM_V_PROGRAM, "syslog-ng", 9);
+  g_snprintf(buf, sizeof(buf), "%d", (int) getpid());
+  log_msg_set_value(msg, LM_V_PID, buf, -1);
+
+  msg->pri = LOG_SYSLOG | LOG_ERR;
+}
 
 void
 msg_format_options_defaults(MsgFormatOptions *options)
@@ -90,6 +109,7 @@ CfgFlagHandler msg_format_flag_handlers[] =
   { "syslog-protocol",            CFH_SET, offsetof(MsgFormatOptions, flags), LP_SYSLOG_PROTOCOL },
   { "assume-utf8",                CFH_SET, offsetof(MsgFormatOptions, flags), LP_ASSUME_UTF8 },
   { "validate-utf8",              CFH_SET, offsetof(MsgFormatOptions, flags), LP_VALIDATE_UTF8 },
+  { "sanitize-utf8",              CFH_SET, offsetof(MsgFormatOptions, flags), LP_SANITIZE_UTF8 },
   { "no-multi-line",              CFH_SET, offsetof(MsgFormatOptions, flags), LP_NO_MULTI_LINE },
   { "store-legacy-msghdr",        CFH_SET, offsetof(MsgFormatOptions, flags), LP_STORE_LEGACY_MSGHDR },
   { "dont-store-legacy-msghdr", CFH_CLEAR, offsetof(MsgFormatOptions, flags), LP_STORE_LEGACY_MSGHDR },
