@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002-2012 BalaBit IT Ltd, Budapest, Hungary
+ * Copyright (c) 2002-2013 BalaBit IT Ltd, Budapest, Hungary
  * Copyright (c) 1998-2012 Balázs Scheidler
  *
  * This program is free software; you can redistribute it and/or modify it
@@ -20,22 +20,20 @@
  * COPYING for details.
  *
  */
-  
-#ifndef AFDFILE_COMMON_H_INCLUDED
-#define AFDFILE_COMMON_H_INCLUDED
+
+#ifndef AFFILE_COMMON_H_INCLUDED
+#define AFFILE_COMMON_H_INCLUDED
 
 #include "file-perms.h"
 
-#define AFFILE_PIPE        0x00000001
-#define AFFILE_NO_EXPAND   0x00000002
-#define AFFILE_TMPL_ESCAPE 0x00000004
-#define AFFILE_CREATE_DIRS 0x00000008
-#define AFFILE_FSYNC       0x00000010
-#define AFFILE_PRIVILEGED  0x00000020
+typedef struct _FileOpenOptions
+{
+  gboolean create_dirs:1,
+           needs_privileges:1,
+           is_pipe:1;
+  gint open_flags;
+} FileOpenOptions;
 
-gboolean affile_open_file(gchar *name, gint flags,
-                          const FilePermOptions *perm_options,
-                          gboolean create_dirs, gboolean privileged,
-                          gboolean is_pipe, gint *fd);
+gboolean affile_open_file(gchar *name, FileOpenOptions *open_opts, FilePermOptions *perm_opts, gint *fd);
 
 #endif
