@@ -65,7 +65,7 @@ log_proto_file_writer_flush(LogProtoClient *s)
 
   if (rc < 0)
     {
-      if (errno != EAGAIN && errno != EINTR)
+      if (errno != EINTR)
         {
           msg_error("I/O error occurred while writing",
                     evt_tag_int("fd", self->super.transport->fd),
@@ -131,6 +131,7 @@ log_proto_file_writer_post(LogProtoClient *s, guchar *msg, gsize msg_len, gboole
   LogProtoFileWriter *self = (LogProtoFileWriter *)s;
   gint rc;
 
+  *consumed = FALSE;
   if (self->buf_count >= self->buf_size)
     {
       rc = log_proto_file_writer_flush(s);
@@ -141,7 +142,6 @@ log_proto_file_writer_post(LogProtoClient *s, guchar *msg, gsize msg_len, gboole
         }
     }
 
-  *consumed = FALSE;
   if (self->partial)
     {
       /* there is still some data from the previous file writing process */
@@ -173,14 +173,17 @@ log_proto_file_writer_post(LogProtoClient *s, guchar *msg, gsize msg_len, gboole
   self->buffer[self->buf_count].iov_len = msg_len;
   ++self->buf_count;
   self->sum_len += msg_len;
-  *consumed = TRUE;
 
   if (self->buf_count == self->buf_size)
     {
       /* we have reached the max buffer size -> we need to write the messages */
-      return log_proto_file_writer_flush(s);
+      LogProtoStatus result = log_proto_file_writer_flush(s);
+      if (result != LPS_SUCCESS)
+        return result;
     }
 
+  *consumed = TRUE;
+  log_proto_client_msg_ack(&self->super, 1);
   return LPS_SUCCESS;
 
 write_error:
