@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2010-2012 BalaBit IT Ltd, Budapest, Hungary
- * Copyright (c) 2010-2012 Gergely Nagy <algernon@balabit.hu>
+ * Copyright (c) 2010-2014 BalaBit IT Ltd, Budapest, Hungary
+ * Copyright (c) 2010-2014 Gergely Nagy <algernon@balabit.hu>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -21,20 +21,24 @@
  *
  */
 
-#include <time.h>
 
 #include "afmongodb.h"
 #include "afmongodb-parser.h"
-#include "plugin.h"
 #include "messages.h"
 #include "misc.h"
-#include "stats.h"
+#include "stats/stats-registry.h"
 #include "nvtable.h"
 #include "logqueue.h"
 #include "value-pairs.h"
 #include "vptransform.h"
+#include "plugin.h"
+#include "plugin-types.h"
+#include "logthrdestdrv.h"
 
 #include "mongo.h"
+#include <time.h>
+
+#define SOCKET_TIMEOUT_FOR_MONGO_CONNECTION_IN_MILLISECS 60000
 
 typedef struct
 {
@@ -44,7 +48,7 @@ typedef struct
 
 typedef struct
 {
-  LogDestDriver super;
+  LogThrDestDriver super;
 
   /* Shared between main/writer; only read by the writer, never
      written */
@@ -56,33 +60,18 @@ typedef struct
   gint port;
 
   gboolean safe_mode;
+  LogTemplateOptions template_options;
 
   gchar *user;
   gchar *password;
 
-  time_t time_reopen;
-
-  StatsCounterItem *dropped_messages;
-  StatsCounterItem *stored_messages;
-
   time_t last_msg_stamp;
 
   ValuePairs *vp;
 
-  /* Thread related stuff; shared */
-  GThread *writer_thread;
-  GMutex *suspend_mutex;
-  GCond *writer_thread_wakeup_cond;
-
-  gboolean writer_thread_terminate;
-  gboolean writer_thread_suspended;
-  GTimeVal writer_thread_suspend_target;
-
-  LogQueue *queue;
-
   /* Writer-only stuff */
+  mongo_sync_conn_recovery_cache *recovery_cache;
   mongo_sync_connection *conn;
-  gint32 seq_num;
 
   gchar *ns;
 
@@ -117,6 +106,8 @@ afmongodb_dd_set_host(LogDriver *d, const gchar *host)
 {
   MongoDBDestDriver *self = (MongoDBDestDriver *)d;
 
+  msg_warning_once("WARNING: Using host() option is deprecated in mongodb driver, please use servers() instead", NULL);
+
   g_free(self->address);
   self->address = g_strdup(host);
 }
@@ -126,6 +117,8 @@ afmongodb_dd_set_port(LogDriver *d, gint port)
 {
   MongoDBDestDriver *self = (MongoDBDestDriver *)d;
 
+  msg_warning_once("WARNING: Using port() option is deprecated in mongodb driver, please use servers() instead", NULL);
+
   self->port = port;
 }
 
@@ -148,6 +141,14 @@ afmongodb_dd_set_path(LogDriver *d, const gchar *path)
   self->port = MONGO_CONN_LOCAL;
 }
 
+LogTemplateOptions *
+afmongodb_dd_get_template_options(LogDriver *s)
+{
+  MongoDBDestDriver *self = (MongoDBDestDriver *) s;
+
+  return &self->template_options;
+}
+
 gboolean
 afmongodb_dd_check_address(LogDriver *d, gboolean local)
 {
@@ -195,7 +196,7 @@ afmongodb_dd_set_value_pairs(LogDriver *d, ValuePairs *vp)
   MongoDBDestDriver *self = (MongoDBDestDriver *)d;
 
   if (self->vp)
-    value_pairs_free (self->vp);
+    value_pairs_unref (self->vp);
   self->vp = vp;
 }
 
@@ -212,8 +213,9 @@ afmongodb_dd_set_safe_mode(LogDriver *d, gboolean state)
  */
 
 static gchar *
-afmongodb_dd_format_stats_instance(MongoDBDestDriver *self)
+afmongodb_dd_format_stats_instance(LogThrDestDriver *d)
 {
+  MongoDBDestDriver *self = (MongoDBDestDriver *)d;
   static gchar persist_name[1024];
 
   if (self->port == MONGO_CONN_LOCAL)
@@ -226,8 +228,9 @@ afmongodb_dd_format_stats_instance(MongoDBDestDriver *self)
 }
 
 static gchar *
-afmongodb_dd_format_persist_name(MongoDBDestDriver *self)
+afmongodb_dd_format_persist_name(LogThrDestDriver *d)
 {
+  MongoDBDestDriver *self = (MongoDBDestDriver *)d;
   static gchar persist_name[1024];
 
   if (self->port == MONGO_CONN_LOCAL)
@@ -240,17 +243,10 @@ afmongodb_dd_format_persist_name(MongoDBDestDriver *self)
 }
 
 static void
-afmongodb_dd_suspend(MongoDBDestDriver *self)
+afmongodb_dd_disconnect(LogThrDestDriver *s)
 {
-  self->writer_thread_suspended = TRUE;
-  g_get_current_time(&self->writer_thread_suspend_target);
-  g_time_val_add(&self->writer_thread_suspend_target,
-		 self->time_reopen * 1000000);
-}
+  MongoDBDestDriver *self = (MongoDBDestDriver *)s;
 
-static void
-afmongodb_dd_disconnect(MongoDBDestDriver *self)
-{
   mongo_sync_disconnect(self->conn);
   self->conn = NULL;
 }
@@ -258,57 +254,31 @@ afmongodb_dd_disconnect(MongoDBDestDriver *self)
 static gboolean
 afmongodb_dd_connect(MongoDBDestDriver *self, gboolean reconnect)
 {
-  GList *l;
-
   if (reconnect && self->conn)
     return TRUE;
 
-  self->conn = mongo_sync_connect(self->address, self->port, FALSE);
+  self->conn = mongo_sync_connect_recovery_cache(self->recovery_cache, FALSE);
+
   if (!self->conn)
     {
-      msg_error ("Error connecting to MongoDB", NULL);
+      msg_error ("Error connecting to MongoDB", evt_tag_str("driver", self->super.super.super.id), NULL);
       return FALSE;
     }
 
+  mongo_connection_set_timeout((mongo_connection*) self->conn, SOCKET_TIMEOUT_FOR_MONGO_CONNECTION_IN_MILLISECS);
+
   mongo_sync_conn_set_safe_mode(self->conn, self->safe_mode);
 
-  l = self->servers;
-  while ((l = g_list_next(l)) != NULL)
-    {
-      gchar *host = NULL;
-      gint port = 27017;
 
-      if (!mongo_util_parse_addr(l->data, &host, &port))
-	{
-	  msg_warning("Cannot parse MongoDB server address, ignoring",
-		      evt_tag_str("address", l->data),
-		      NULL);
-	  continue;
-	}
-      mongo_sync_conn_seed_add (self->conn, host, port);
-      msg_verbose("Added MongoDB server seed",
-		  evt_tag_str("host", host),
-		  evt_tag_int("port", port),
-		  NULL);
-      g_free(host);
-    }
-
-  /*
   if (self->user || self->password)
     {
-      if (!self->user || !self->password)
-	{
-	  msg_error("Neither the username, nor the password can be empty", NULL);
-	  return FALSE;
-	}
-
-      if (mongo_cmd_authenticate(&self->mongo_conn, self->db, self->user, self->password) != 1)
-	{
-	  msg_error("MongoDB authentication failed", NULL);
-	  return FALSE;
-	}
+      if (!mongo_sync_cmd_authenticate (self->conn, self->db,
+                                        self->user, self->password))
+        {
+          msg_error("MongoDB authentication failed", evt_tag_str("driver", self->super.super.super.id), NULL);
+          return FALSE;
+        }
     }
-  */
 
   return TRUE;
 }
@@ -338,12 +308,13 @@ afmongodb_vp_obj_end(const gchar *name,
                      const gchar *prev, gpointer *prev_data,
                      gpointer user_data)
 {
+  MongoDBDestDriver *self = (MongoDBDestDriver *)user_data;
   bson *root;
 
   if (prev_data)
     root = (bson *)*prev_data;
   else
-    root = (bson *)user_data;
+    root = self->bson;
 
   if (prefix_data)
     {
@@ -358,94 +329,206 @@ afmongodb_vp_obj_end(const gchar *name,
 
 static gboolean
 afmongodb_vp_process_value(const gchar *name, const gchar *prefix,
-                           const gchar *value,
+                           TypeHint type, const gchar *value,
                            gpointer *prefix_data, gpointer user_data)
 {
   bson *o;
+  MongoDBDestDriver *self = (MongoDBDestDriver *)user_data;
+  gboolean fallback = self->template_options.on_error & ON_ERROR_FALLBACK_TO_STRING;
 
   if (prefix_data)
     o = (bson *)*prefix_data;
   else
-    o = (bson *)user_data;
+    o = self->bson;
 
-  bson_append_string (o, name, value, -1);
+  switch (type)
+    {
+    case TYPE_HINT_BOOLEAN:
+      {
+        gboolean b;
+
+        if (type_cast_to_boolean (value, &b, NULL))
+          bson_append_boolean (o, name, b);
+        else
+          {
+            gboolean r = type_cast_drop_helper(self->template_options.on_error,
+                                               value, "boolean");
+
+            if (fallback)
+              bson_append_string (o, name, value, -1);
+            else
+              return r;
+          }
+        break;
+      }
+    case TYPE_HINT_INT32:
+      {
+        gint32 i;
+
+        if (type_cast_to_int32 (value, &i, NULL))
+          bson_append_int32 (o, name, i);
+        else
+          {
+            gboolean r = type_cast_drop_helper(self->template_options.on_error,
+                                               value, "int32");
+
+            if (fallback)
+              bson_append_string (o, name, value, -1);
+            else
+              return r;
+          }
+        break;
+      }
+    case TYPE_HINT_INT64:
+      {
+        gint64 i;
+
+        if (type_cast_to_int64 (value, &i, NULL))
+          bson_append_int64 (o, name, i);
+        else
+          {
+            gboolean r = type_cast_drop_helper(self->template_options.on_error,
+                                               value, "int64");
+
+            if (fallback)
+              bson_append_string(o, name, value, -1);
+            else
+              return r;
+          }
+
+        break;
+      }
+    case TYPE_HINT_DOUBLE:
+      {
+        gdouble d;
+
+        if (type_cast_to_double (value, &d, NULL))
+          bson_append_double (o, name, d);
+        else
+          {
+            gboolean r = type_cast_drop_helper(self->template_options.on_error,
+                                               value, "double");
+            if (fallback)
+              bson_append_string(o, name, value, -1);
+            else
+              return r;
+          }
+
+        break;
+      }
+    case TYPE_HINT_DATETIME:
+      {
+        guint64 i;
+
+        if (type_cast_to_datetime_int (value, &i, NULL))
+          bson_append_utc_datetime (o, name, (gint64)i);
+        else
+          {
+            gboolean r = type_cast_drop_helper(self->template_options.on_error,
+                                               value, "datetime");
+
+            if (fallback)
+              bson_append_string(o, name, value, -1);
+            else
+              return r;
+          }
+
+        break;
+      }
+    case TYPE_HINT_STRING:
+    case TYPE_HINT_LITERAL:
+      bson_append_string (o, name, value, -1);
+      break;
+    default:
+      return TRUE;
+    }
 
   return FALSE;
 }
 
-static gboolean
-afmongodb_worker_insert (MongoDBDestDriver *self)
+static void
+afmongodb_worker_retry_over_message(LogThrDestDriver *s, LogMessage *msg)
 {
+  MongoDBDestDriver *self = (MongoDBDestDriver *)s;
+
+  msg_error("Multiple failures while inserting this record into the database, "
+            "message dropped",
+            evt_tag_str("driver", self->super.super.super.id),
+            evt_tag_int("number_of_retries", s->retries.max),
+            evt_tag_value_pairs("message", self->vp, msg,
+                                self->super.seq_num,
+                                LTZ_SEND, &self->template_options),
+            NULL);
+}
+
+static worker_insert_result_t
+afmongodb_worker_insert (LogThrDestDriver *s, LogMessage *msg)
+{
+  MongoDBDestDriver *self = (MongoDBDestDriver *)s;
   gboolean success;
-  guint8 *oid;
-  LogMessage *msg;
-  LogPathOptions path_options = LOG_PATH_OPTIONS_INIT;
+  gboolean drop_silently = self->template_options.on_error & ON_ERROR_SILENT;
 
-  afmongodb_dd_connect(self, TRUE);
-
-  success = log_queue_pop_head(self->queue, &msg, &path_options, FALSE, FALSE);
-  if (!success)
-    return TRUE;
-
-  msg_set_context(msg);
+  if (!afmongodb_dd_connect(self, TRUE))
+    return WORKER_INSERT_RESULT_NOT_CONNECTED;
 
   bson_reset (self->bson);
 
-  oid = mongo_util_oid_new_with_time (self->last_msg_stamp, self->seq_num);
-  bson_append_oid (self->bson, "_id", oid);
-  g_free (oid);
-
-  value_pairs_walk(self->vp,
-                   afmongodb_vp_obj_start,
-                   afmongodb_vp_process_value,
-                   afmongodb_vp_obj_end,
-                   msg, self->seq_num, self->bson);
+  success = value_pairs_walk(self->vp,
+                             afmongodb_vp_obj_start,
+                             afmongodb_vp_process_value,
+                             afmongodb_vp_obj_end,
+                             msg, self->super.seq_num,
+                             LTZ_SEND,
+                             &self->template_options,
+                             self);
   bson_finish (self->bson);
 
-  if (!mongo_sync_cmd_insert_n(self->conn, self->ns, 1,
-                               (const bson **)&self->bson))
+  if (!success)
     {
-      msg_error("Network error while inserting into MongoDB",
-                evt_tag_int("time_reopen", self->time_reopen),
-                NULL);
-      success = FALSE;
-    }
-
-  msg_set_context(NULL);
-
-  if (success)
-    {
-      stats_counter_inc(self->stored_messages);
-      step_sequence_number(&self->seq_num);
-      log_msg_ack(msg, &path_options);
-      log_msg_unref(msg);
+      if (!drop_silently)
+        {
+          msg_error("Failed to format message for MongoDB, dropping message",
+                    evt_tag_value_pairs("message", self->vp, msg,
+                                        self->super.seq_num,
+                                        LTZ_SEND, &self->template_options),
+                    evt_tag_str("driver", self->super.super.super.id),
+                    NULL);
+        }
+      return WORKER_INSERT_RESULT_DROP;
     }
   else
     {
-      log_queue_push_head(self->queue, msg, &path_options);
+      msg_debug("Outgoing message to MongoDB destination",
+                evt_tag_value_pairs("message", self->vp, msg,
+                                    self->super.seq_num,
+                                    LTZ_SEND, &self->template_options),
+                evt_tag_str("driver", self->super.super.super.id),
+                NULL);
+      if (!mongo_sync_cmd_insert_n(self->conn, self->ns, 1,
+                                   (const bson **)&self->bson))
+        {
+          msg_error("Network error while inserting into MongoDB",
+                    evt_tag_int("time_reopen", self->super.time_reopen),
+                    evt_tag_str("reason", mongo_sync_conn_get_last_error(self->conn)),
+                    evt_tag_str("driver", self->super.super.super.id),
+                    NULL);
+          success = FALSE;
+        }
     }
 
-  return success;
+  if (!success && (errno == ENOTCONN))
+    return WORKER_INSERT_RESULT_NOT_CONNECTED;
+
+  if (!success)
+    return WORKER_INSERT_RESULT_ERROR;
+
+  return WORKER_INSERT_RESULT_SUCCESS;
 }
 
 static void
-afmongodb_dd_message_became_available_in_the_queue(gpointer user_data)
+afmongodb_worker_thread_init(LogThrDestDriver *d)
 {
-  MongoDBDestDriver *self = (MongoDBDestDriver *) user_data;
-
-  g_mutex_lock(self->suspend_mutex);
-  g_cond_signal(self->writer_thread_wakeup_cond);
-  g_mutex_unlock(self->suspend_mutex);
-}
-
-static gpointer
-afmongodb_worker_thread (gpointer arg)
-{
-  MongoDBDestDriver *self = (MongoDBDestDriver *)arg;
-
-  msg_debug ("Worker thread started",
-	     evt_tag_str("driver", self->super.super.id),
-	     NULL);
+  MongoDBDestDriver *self = (MongoDBDestDriver *)d;
 
   afmongodb_dd_connect(self, FALSE);
 
@@ -454,95 +537,63 @@ afmongodb_worker_thread (gpointer arg)
   self->current_value = g_string_sized_new(256);
 
   self->bson = bson_new_sized(4096);
+}
 
-  while (!self->writer_thread_terminate)
-    {
-      g_mutex_lock(self->suspend_mutex);
-      if (self->writer_thread_suspended)
-	{
-	  g_cond_timed_wait(self->writer_thread_wakeup_cond,
-			    self->suspend_mutex,
-			    &self->writer_thread_suspend_target);
-	  self->writer_thread_suspended = FALSE;
-	  g_mutex_unlock(self->suspend_mutex);
-	}
-      else if (!log_queue_check_items(self->queue, NULL, afmongodb_dd_message_became_available_in_the_queue, self, NULL))
-	{
-	  g_cond_wait(self->writer_thread_wakeup_cond, self->suspend_mutex);
-	  g_mutex_unlock(self->suspend_mutex);
-	}
-      else
-        g_mutex_unlock(self->suspend_mutex);
-
-      if (self->writer_thread_terminate)
-	break;
-
-      if (!afmongodb_worker_insert (self))
-	{
-	  afmongodb_dd_disconnect(self);
-	  afmongodb_dd_suspend(self);
-	}
-    }
-
-  afmongodb_dd_disconnect(self);
+static void
+afmongodb_worker_thread_deinit(LogThrDestDriver *d)
+{
+  MongoDBDestDriver *self = (MongoDBDestDriver *)d;
 
   g_free (self->ns);
   g_string_free (self->current_value, TRUE);
 
   bson_free (self->bson);
-
-  msg_debug ("Worker thread finished",
-	     evt_tag_str("driver", self->super.super.id),
-	     NULL);
-
-  return NULL;
 }
 
 /*
  * Main thread
  */
 
-static void
-afmongodb_dd_start_thread (MongoDBDestDriver *self)
+static gboolean
+afmongodb_dd_check_auth_options(MongoDBDestDriver *self)
 {
-  self->writer_thread = create_worker_thread(afmongodb_worker_thread, self, TRUE, NULL);
+  if (self->user || self->password)
+    {
+      if (!self->user || !self->password)
+        {
+          msg_error("Neither the username, nor the password can be empty", NULL);
+          return FALSE;
+        }
+    }
+  return TRUE;
 }
 
 static void
-afmongodb_dd_stop_thread (MongoDBDestDriver *self)
+afmongodb_dd_init_value_pairs_dot_to_underscore_transformation(MongoDBDestDriver *self)
 {
-  self->writer_thread_terminate = TRUE;
-  g_mutex_lock(self->suspend_mutex);
-  g_cond_signal(self->writer_thread_wakeup_cond);
-  g_mutex_unlock(self->suspend_mutex);
-  g_thread_join(self->writer_thread);
-}
+  ValuePairsTransformSet *vpts;
+
+ /* Always replace a leading dot with an underscore. */
+  vpts = value_pairs_transform_set_new(".*");
+  value_pairs_transform_set_add_func(vpts, value_pairs_new_transform_replace_prefix(".", "_"));
+  value_pairs_add_transforms(self->vp, vpts);
+};
 
 static gboolean
 afmongodb_dd_init(LogPipe *s)
 {
   MongoDBDestDriver *self = (MongoDBDestDriver *)s;
   GlobalConfig *cfg = log_pipe_get_config(s);
-  ValuePairsTransformSet *vpts;
 
   if (!log_dest_driver_init_method(s))
     return FALSE;
 
-  if (cfg)
-    self->time_reopen = cfg->time_reopen;
+  log_template_options_init(&self->template_options, cfg);
 
-  if (!self->vp)
-    {
-      self->vp = value_pairs_new();
-      value_pairs_add_scope(self->vp, "selected-macros");
-      value_pairs_add_scope(self->vp, "nv-pairs");
-    }
-
-  /* Always replace a leading dot with an underscore. */
-  vpts = value_pairs_transform_set_new(".*");
-  value_pairs_transform_set_add_func(vpts, value_pairs_new_transform_replace(".", "_"));
-  value_pairs_add_transforms(self->vp, vpts);
+  if (!afmongodb_dd_check_auth_options(self))
+    return FALSE;
 
+  afmongodb_dd_init_value_pairs_dot_to_underscore_transformation(self);
   if (self->port != MONGO_CONN_LOCAL)
     {
       if (self->address)
@@ -553,8 +604,37 @@ afmongodb_dd_init(LogPipe *s)
           g_free (self->address);
         }
 
-      if (!self->servers)
-        afmongodb_dd_set_servers((LogDriver *)self, g_list_append (NULL, g_strdup ("127.0.0.1:27017")));
+      if (self->servers)
+        {
+          GList *l;
+
+          for (l=self->servers; l; l = g_list_next(l))
+            {
+              gchar *host = NULL;
+              gint port = 27017;
+
+              if (!mongo_util_parse_addr(l->data, &host, &port))
+                {
+                  msg_warning("Cannot parse MongoDB server address, ignoring",
+                              evt_tag_str("address", l->data),
+                              evt_tag_str("driver", self->super.super.super.id),
+                              NULL);
+                  continue;
+                }
+              mongo_sync_conn_recovery_cache_seed_add (self->recovery_cache, host, port);
+              msg_verbose("Added MongoDB server seed",
+                          evt_tag_str("host", host),
+                          evt_tag_int("port", port),
+                          evt_tag_str("driver", self->super.super.super.id),
+                          NULL);
+              g_free(host);
+            }
+        }
+      else
+        {
+          afmongodb_dd_set_servers((LogDriver *)self, g_list_append (NULL, g_strdup ("127.0.0.1:27017")));
+          mongo_sync_conn_recovery_cache_seed_add (self->recovery_cache, "127.0.0.1", 27017);
+        }
 
       self->address = NULL;
       self->port = 27017;
@@ -564,16 +644,22 @@ afmongodb_dd_init(LogPipe *s)
         {
           msg_error("Cannot parse the primary host",
                     evt_tag_str("primary", g_list_nth_data(self->servers, 0)),
+                    evt_tag_str("driver", self->super.super.super.id),
                     NULL);
           return FALSE;
         }
     }
+  else
+    {
+      mongo_sync_conn_recovery_cache_seed_add (self->recovery_cache, self->address, self->port);
+    }
 
   if (self->port == MONGO_CONN_LOCAL)
     msg_verbose("Initializing MongoDB destination",
                 evt_tag_str("address", self->address),
                 evt_tag_str("database", self->db),
                 evt_tag_str("collection", self->coll),
+                evt_tag_str("driver", self->super.super.super.id),
                 NULL);
   else
     msg_verbose("Initializing MongoDB destination",
@@ -581,46 +667,10 @@ afmongodb_dd_init(LogPipe *s)
                 evt_tag_int("port", self->port),
                 evt_tag_str("database", self->db),
                 evt_tag_str("collection", self->coll),
+                evt_tag_str("driver", self->super.super.super.id),
                 NULL);
 
-  self->queue = log_dest_driver_acquire_queue(&self->super, afmongodb_dd_format_persist_name(self));
-
-  stats_lock();
-  stats_register_counter(0, SCS_MONGODB | SCS_DESTINATION, self->super.super.id,
-			 afmongodb_dd_format_stats_instance(self),
-			 SC_TYPE_STORED, &self->stored_messages);
-  stats_register_counter(0, SCS_MONGODB | SCS_DESTINATION, self->super.super.id,
-			 afmongodb_dd_format_stats_instance(self),
-			 SC_TYPE_DROPPED, &self->dropped_messages);
-  stats_unlock();
-
-  log_queue_set_counters(self->queue, self->stored_messages, self->dropped_messages);
-  afmongodb_dd_start_thread(self);
-
-  return TRUE;
-}
-
-static gboolean
-afmongodb_dd_deinit(LogPipe *s)
-{
-  MongoDBDestDriver *self = (MongoDBDestDriver *)s;
-
-  afmongodb_dd_stop_thread(self);
-  log_queue_reset_parallel_push(self->queue);
-
-  log_queue_set_counters(self->queue, NULL, NULL);
-  stats_lock();
-  stats_unregister_counter(SCS_MONGODB | SCS_DESTINATION, self->super.super.id,
-			   afmongodb_dd_format_stats_instance(self),
-			   SC_TYPE_STORED, &self->stored_messages);
-  stats_unregister_counter(SCS_MONGODB | SCS_DESTINATION, self->super.super.id,
-			   afmongodb_dd_format_stats_instance(self),
-			   SC_TYPE_DROPPED, &self->dropped_messages);
-  stats_unlock();
-  if (!log_dest_driver_deinit_method(s))
-    return FALSE;
-
-  return TRUE;
+  return log_threaded_dest_driver_start(s);
 }
 
 static void
@@ -628,11 +678,7 @@ afmongodb_dd_free(LogPipe *d)
 {
   MongoDBDestDriver *self = (MongoDBDestDriver *)d;
 
-  g_mutex_free(self->suspend_mutex);
-  g_cond_free(self->writer_thread_wakeup_cond);
-
-  if (self->queue)
-    log_queue_unref(self->queue);
+  log_template_options_destroy(&self->template_options);
 
   g_free(self->db);
   g_free(self->coll);
@@ -641,25 +687,19 @@ afmongodb_dd_free(LogPipe *d)
   g_free(self->address);
   string_list_free(self->servers);
   if (self->vp)
-    value_pairs_free(self->vp);
-  log_dest_driver_free(d);
+    value_pairs_unref(self->vp);
+
+  mongo_sync_conn_recovery_cache_free(self->recovery_cache);
+  self->recovery_cache = NULL;
+  log_threaded_dest_driver_free(d);
 }
 
 static void
-afmongodb_dd_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options, gpointer user_data)
+afmongodb_dd_queue_method(LogThrDestDriver *d)
 {
-  MongoDBDestDriver *self = (MongoDBDestDriver *)s;
-  LogPathOptions local_options;
-
-  if (!path_options->flow_control_requested)
-    path_options = log_msg_break_ack(msg, path_options, &local_options);
+  MongoDBDestDriver *self = (MongoDBDestDriver *)d;
 
   self->last_msg_stamp = cached_g_current_time_sec ();
-
-  log_msg_add_ack(msg, path_options);
-  log_queue_push_tail(self->queue, log_msg_ref(msg), path_options);
-
-  log_dest_driver_queue_method(s, msg, path_options, user_data);
 }
 
 /*
@@ -667,26 +707,37 @@ afmongodb_dd_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_optio
  */
 
 LogDriver *
-afmongodb_dd_new(void)
+afmongodb_dd_new(GlobalConfig *cfg)
 {
   MongoDBDestDriver *self = g_new0(MongoDBDestDriver, 1);
 
   mongo_util_oid_init (0);
 
-  log_dest_driver_init_instance(&self->super);
-  self->super.super.super.init = afmongodb_dd_init;
-  self->super.super.super.deinit = afmongodb_dd_deinit;
-  self->super.super.super.queue = afmongodb_dd_queue;
-  self->super.super.super.free_fn = afmongodb_dd_free;
+  log_threaded_dest_driver_init_instance(&self->super, cfg);
+
+  self->super.super.super.super.init = afmongodb_dd_init;
+  self->super.super.super.super.free_fn = afmongodb_dd_free;
+  self->super.queue_method = afmongodb_dd_queue_method;
+
+  self->super.worker.thread_init = afmongodb_worker_thread_init;
+  self->super.worker.thread_deinit = afmongodb_worker_thread_deinit;
+  self->super.worker.disconnect = afmongodb_dd_disconnect;
+  self->super.worker.insert = afmongodb_worker_insert;
+  self->super.format.stats_instance = afmongodb_dd_format_stats_instance;
+  self->super.format.persist_name = afmongodb_dd_format_persist_name;
+  self->super.stats_source = SCS_MONGODB;
+  self->super.messages.retry_over = afmongodb_worker_retry_over_message;
 
   afmongodb_dd_set_database((LogDriver *)self, "syslog");
   afmongodb_dd_set_collection((LogDriver *)self, "messages");
   afmongodb_dd_set_safe_mode((LogDriver *)self, FALSE);
 
-  init_sequence_number(&self->seq_num);
+  self->safe_mode = TRUE;
 
-  self->writer_thread_wakeup_cond = g_cond_new();
-  self->suspend_mutex = g_mutex_new();
+  log_template_options_defaults(&self->template_options);
+  afmongodb_dd_set_value_pairs(&self->super.super.super, value_pairs_new_default(cfg));
+
+  self->recovery_cache = mongo_sync_conn_recovery_cache_new();
 
   return (LogDriver *)self;
 }
