@@ -57,6 +57,7 @@
      KW_TCP_KEEPALIVE_PROBES = 20018,
      KW_TCP_KEEPALIVE_INTVL = 20019,
      KW_SPOOF_SOURCE = 20020,
+     KW_PASS_UNIX_CREDENTIALS = 10231,
      KW_KEEP_ALIVE = 20021,
      KW_MAX_CONNECTIONS = 20022,
      KW_LOCALIP = 20023,
@@ -72,6 +73,7 @@
      KW_TRUSTED_KEYS = 20033,
      KW_TRUSTED_DN = 20034,
      KW_CIPHER_SUITE = 20035,
+     KW_SSL_OPTIONS = 20036,
      LL_CONTEXT_ROOT = 1,
      LL_CONTEXT_DESTINATION = 2,
      LL_CONTEXT_SOURCE = 3,
@@ -117,6 +119,8 @@
      KW_QUOTES = 10051,
      KW_QUOTE_PAIRS = 10052,
      KW_NULL = 10053,
+     KW_CHARS = 10054,
+     KW_STRINGS = 10055,
      KW_SYSLOG = 10060,
      KW_MARK_FREQ = 10071,
      KW_STATS_FREQ = 10072,
@@ -145,6 +149,7 @@
      KW_DNS_CACHE_HOSTS = 10132,
      KW_PERSIST_ONLY = 10140,
      KW_USE_RCPTID = 10141,
+     KW_USE_UNIQID = 10142,
      KW_TZ_CONVERT = 10150,
      KW_TS_FORMAT = 10151,
      KW_FRAC_DIGITS = 10152,
@@ -177,6 +182,7 @@
      KW_DIR_PERM = 10262,
      KW_TEMPLATE = 10270,
      KW_TEMPLATE_ESCAPE = 10271,
+     KW_TEMPLATE_FUNCTION = 10272,
      KW_DEFAULT_FACILITY = 10300,
      KW_DEFAULT_LEVEL = 10301,
      KW_PORT = 10323,
@@ -188,6 +194,7 @@
      KW_MESSAGE = 10354,
      KW_NETMASK = 10355,
      KW_TAGS = 10356,
+     KW_NETMASK6 = 10357,
      KW_VALUE = 10361,
      KW_REWRITE = 10370,
      KW_SET = 10371,
@@ -241,6 +248,7 @@
 #define KW_TCP_KEEPALIVE_PROBES 20018
 #define KW_TCP_KEEPALIVE_INTVL 20019
 #define KW_SPOOF_SOURCE 20020
+#define KW_PASS_UNIX_CREDENTIALS 10231
 #define KW_KEEP_ALIVE 20021
 #define KW_MAX_CONNECTIONS 20022
 #define KW_LOCALIP 20023
@@ -256,6 +264,7 @@
 #define KW_TRUSTED_KEYS 20033
 #define KW_TRUSTED_DN 20034
 #define KW_CIPHER_SUITE 20035
+#define KW_SSL_OPTIONS 20036
 #define LL_CONTEXT_ROOT 1
 #define LL_CONTEXT_DESTINATION 2
 #define LL_CONTEXT_SOURCE 3
@@ -301,6 +310,8 @@
 #define KW_QUOTES 10051
 #define KW_QUOTE_PAIRS 10052
 #define KW_NULL 10053
+#define KW_CHARS 10054
+#define KW_STRINGS 10055
 #define KW_SYSLOG 10060
 #define KW_MARK_FREQ 10071
 #define KW_STATS_FREQ 10072
@@ -329,6 +340,7 @@
 #define KW_DNS_CACHE_HOSTS 10132
 #define KW_PERSIST_ONLY 10140
 #define KW_USE_RCPTID 10141
+#define KW_USE_UNIQID 10142
 #define KW_TZ_CONVERT 10150
 #define KW_TS_FORMAT 10151
 #define KW_FRAC_DIGITS 10152
@@ -361,6 +373,7 @@
 #define KW_DIR_PERM 10262
 #define KW_TEMPLATE 10270
 #define KW_TEMPLATE_ESCAPE 10271
+#define KW_TEMPLATE_FUNCTION 10272
 #define KW_DEFAULT_FACILITY 10300
 #define KW_DEFAULT_LEVEL 10301
 #define KW_PORT 10323
@@ -372,6 +385,7 @@
 #define KW_MESSAGE 10354
 #define KW_NETMASK 10355
 #define KW_TAGS 10356
+#define KW_NETMASK6 10357
 #define KW_VALUE 10361
 #define KW_REWRITE 10370
 #define KW_SET 10371
