@@ -33,9 +33,13 @@ typedef struct _AFUnixSourceDriver
   AFSocketSourceDriver super;
   gchar *filename;
   FilePermOptions file_perm_options;
+  gboolean pass_unix_credentials;
+  gboolean create_dirs;
 } AFUnixSourceDriver;
 
 AFUnixSourceDriver *afunix_sd_new_stream(gchar *filename, GlobalConfig *cfg);
 AFUnixSourceDriver *afunix_sd_new_dgram(gchar *filename, GlobalConfig *cfg);
+void afunix_sd_set_pass_unix_credentials(AFUnixSourceDriver *self, gboolean pass);
+void afunix_sd_set_create_dirs(AFUnixSourceDriver *self, gboolean create_dirs);
 
 #endif
