@@ -523,14 +523,8 @@ pdbtool_match(int argc, char *argv[])
       if (G_UNLIKELY(debug_pattern))
         {
           const gchar *msg_string;
-          PDBRule *rule;
-          PDBInput input;
 
-          rule = pdb_rule_set_lookup(patterndb->ruleset,
-                                     PDB_INPUT_WRAP_MESSAGE(&input, msg),
-                                     dbg_list);
-          if (rule)
-            pdb_rule_unref(rule);
+          pattern_db_debug_ruleset(patterndb, msg, dbg_list);
 
           msg_string = log_msg_get_value(msg, LM_V_MESSAGE, NULL);
           pos = 0;
@@ -610,9 +604,7 @@ pdbtool_match(int argc, char *argv[])
         }
       else
         {
-          PDBInput input;
-
-          pattern_db_process(patterndb, PDB_INPUT_WRAP_MESSAGE(&input, msg));
+          pattern_db_process(patterndb, msg);
         }
 
       if (G_LIKELY(proto))
@@ -690,6 +682,42 @@ pdbtool_test_value(LogMessage *msg, const gchar *name, const gchar *test_value)
   return ret;
 }
 
+static void
+pdbtool_test_find_conflicts(PatternDB *patterndb, LogMessage *msg)
+{
+  const gchar *program;
+  const gchar *message;
+  RNode *node;
+
+  program = log_msg_get_value(msg, LM_V_PROGRAM, NULL);
+  message = log_msg_get_value(msg, LM_V_MESSAGE, NULL);
+
+  node = r_find_node(patterndb->ruleset->programs, (gchar *) program, strlen(program), NULL);
+  if (node)
+    {
+      PDBProgram *program_rules = (PDBProgram *) node->value;
+      gchar **matching_ids;
+      gint matching_ids_len;
+
+      matching_ids = r_find_all_applicable_nodes(program_rules->rules, (guint8 *) message, strlen(message), (RNodeGetValueFunc) pdb_rule_get_name);
+      matching_ids_len = g_strv_length(matching_ids);
+
+      if (matching_ids_len > 1)
+        {
+          gint i;
+
+          printf(" Rule conflict! Multiple rules match this message, list of IDs follow\n");
+          for (i = 0; matching_ids[i]; i++)
+            {
+              printf("  %s\n", matching_ids[i]);
+            }
+        }
+
+
+      g_strfreev(matching_ids);
+    }
+}
+
 static gint
 pdbtool_test(int argc, char *argv[])
 {
@@ -746,12 +774,11 @@ pdbtool_test(int argc, char *argv[])
 
           if (example->message && example->program)
             {
-              PDBInput input;
-
               if (test_ruleid)
                 {
                   if (strcmp(example->rule->rule_id, test_ruleid) != 0)
                     {
+                      pdb_example_free(example);
                       examples = g_list_delete_link(examples, examples);
                       continue;
                     }
@@ -766,7 +793,8 @@ pdbtool_test(int argc, char *argv[])
 
               printf("Testing message: program='%s' message='%s'\n", example->program, example->message);
 
-              pattern_db_process(patterndb, PDB_INPUT_WRAP_MESSAGE(&input, msg));
+              pdbtool_test_find_conflicts(patterndb, msg);
+              pattern_db_process(patterndb, msg);
 
               if (!pdbtool_test_value(msg, ".classifier.rule_id", example->rule->rule_id))
                 {
@@ -798,6 +826,7 @@ pdbtool_test(int argc, char *argv[])
               printf("NOT Testing message as message is unset: program='%s'\n", example->program);
             }
 
+          pdb_example_free(example);
           examples = g_list_delete_link(examples, examples);
         }
 
@@ -872,7 +901,7 @@ pdbtool_dump(int argc, char *argv[])
     pdbtool_walk_tree(patterndb->ruleset->programs, 0, TRUE);
   else if (match_program)
     {
-      RNode *ruleset = r_find_node(patterndb->ruleset->programs, g_strdup(match_program), g_strdup(match_program), strlen(match_program), NULL);
+      RNode *ruleset = r_find_node(patterndb->ruleset->programs, match_program, strlen(match_program), NULL);
       if (ruleset && ruleset->value)
         {
           RNode *root = ((PDBProgram *) ruleset->value)->rules;
@@ -973,7 +1002,7 @@ pdbtool_dictionary(int argc, char *argv[])
 
   if (match_program)
     {
-      RNode *ruleset = r_find_node(rule_set.programs, g_strdup(match_program), g_strdup(match_program), strlen(match_program), NULL);
+      RNode *ruleset = r_find_node(rule_set.programs, match_program, strlen(match_program), NULL);
       if (ruleset && ruleset->value)
         pdbtool_dictionary_walk(((PDBProgram *)ruleset->value)->rules, (gchar *)ruleset->key);
     }
