@@ -95,6 +95,8 @@
      KW_QUOTES = 10051,
      KW_QUOTE_PAIRS = 10052,
      KW_NULL = 10053,
+     KW_CHARS = 10054,
+     KW_STRINGS = 10055,
      KW_SYSLOG = 10060,
      KW_MARK_FREQ = 10071,
      KW_STATS_FREQ = 10072,
@@ -123,6 +125,7 @@
      KW_DNS_CACHE_HOSTS = 10132,
      KW_PERSIST_ONLY = 10140,
      KW_USE_RCPTID = 10141,
+     KW_USE_UNIQID = 10142,
      KW_TZ_CONVERT = 10150,
      KW_TS_FORMAT = 10151,
      KW_FRAC_DIGITS = 10152,
@@ -134,6 +137,7 @@
      KW_HOST_OVERRIDE = 10166,
      KW_THROTTLE = 10170,
      KW_THREADED = 10171,
+     KW_PASS_UNIX_CREDENTIALS = 10231,
      KW_FLAGS = 10190,
      KW_PAD_SIZE = 10200,
      KW_TIME_ZONE = 10201,
@@ -155,6 +159,7 @@
      KW_DIR_PERM = 10262,
      KW_TEMPLATE = 10270,
      KW_TEMPLATE_ESCAPE = 10271,
+     KW_TEMPLATE_FUNCTION = 10272,
      KW_DEFAULT_FACILITY = 10300,
      KW_DEFAULT_LEVEL = 10301,
      KW_PORT = 10323,
@@ -166,6 +171,7 @@
      KW_MESSAGE = 10354,
      KW_NETMASK = 10355,
      KW_TAGS = 10356,
+     KW_NETMASK6 = 10357,
      KW_VALUE = 10361,
      KW_REWRITE = 10370,
      KW_SET = 10371,
@@ -246,6 +252,8 @@
 #define KW_QUOTES 10051
 #define KW_QUOTE_PAIRS 10052
 #define KW_NULL 10053
+#define KW_CHARS 10054
+#define KW_STRINGS 10055
 #define KW_SYSLOG 10060
 #define KW_MARK_FREQ 10071
 #define KW_STATS_FREQ 10072
@@ -274,6 +282,7 @@
 #define KW_DNS_CACHE_HOSTS 10132
 #define KW_PERSIST_ONLY 10140
 #define KW_USE_RCPTID 10141
+#define KW_USE_UNIQID 10142
 #define KW_TZ_CONVERT 10150
 #define KW_TS_FORMAT 10151
 #define KW_FRAC_DIGITS 10152
@@ -285,6 +294,7 @@
 #define KW_HOST_OVERRIDE 10166
 #define KW_THROTTLE 10170
 #define KW_THREADED 10171
+#define KW_PASS_UNIX_CREDENTIALS 10231
 #define KW_FLAGS 10190
 #define KW_PAD_SIZE 10200
 #define KW_TIME_ZONE 10201
@@ -306,6 +316,7 @@
 #define KW_DIR_PERM 10262
 #define KW_TEMPLATE 10270
 #define KW_TEMPLATE_ESCAPE 10271
+#define KW_TEMPLATE_FUNCTION 10272
 #define KW_DEFAULT_FACILITY 10300
 #define KW_DEFAULT_LEVEL 10301
 #define KW_PORT 10323
@@ -317,6 +328,7 @@
 #define KW_MESSAGE 10354
 #define KW_NETMASK 10355
 #define KW_TAGS 10356
+#define KW_NETMASK6 10357
 #define KW_VALUE 10361
 #define KW_REWRITE 10370
 #define KW_SET 10371
