@@ -28,6 +28,7 @@
 #include "misc.h"
 #include "cfg.h"
 #include "str-format.h"
+#include "utf8utils.h"
 
 #include <regex.h>
 #include <ctype.h>
@@ -384,17 +385,15 @@ log_msg_parse_date(LogMessage *self, const guchar **data, gint *length, guint pa
               src += i;
             }
 
-          /* detect if the message is coming from last year. If its
-           * month is at least one larger than the current month. This
-           * handles both clocks that are in the future, or in the
-           * past:
-           *   in January we receive a message from December (past) => last year
-           *   in January we receive a message from February (future) => same year
-           *   in December we receive a message from January (future) => next year
+          /* detect if the message is coming from last year. If current
+           * month is January, and the message comes from December, the 
+	   * year of the message is inferred to be last year.
+	   * Conversely, if current month is December, and message comes
+	   * from January, year of message is inferred to be next year.
            */
-          if (tm.tm_mon > nowtm.tm_mon + 1)
+          if (tm.tm_mon == 11 &&  nowtm.tm_mon == 0)
             tm.tm_year--;
-          if (tm.tm_mon < nowtm.tm_mon - 1)
+          else if (tm.tm_mon == 0 && nowtm.tm_mon == 11)
             tm.tm_year++;
 
           /* NOTE: no timezone information in the message, assume it is local time */
@@ -943,9 +942,33 @@ log_msg_parse_legacy(const MsgFormatOptions *parse_options,
       self->timestamps[LM_TS_STAMP] = self->timestamps[LM_TS_RECVD];
     }
 
-  log_msg_set_value(self, LM_V_MESSAGE, (gchar *) src, left);
-  if ((parse_options->flags & LP_VALIDATE_UTF8) && g_utf8_validate((gchar *) src, left, NULL))
-    self->flags |= LF_UTF8;
+  if (parse_options->flags & LP_SANITIZE_UTF8 && !g_utf8_validate((gchar *) src, left, NULL))
+    {
+      GString sanitized_message;
+      gchar buf[left * 6 + 1];
+
+      /* avoid GString allocation */
+      sanitized_message.str = buf;
+      sanitized_message.len = 0;
+      sanitized_message.allocated_len = sizeof(buf);
+
+      append_unsafe_utf8_as_escaped_binary(&sanitized_message, (const gchar *) src, NULL);
+
+      /* MUST NEVER BE REALLOCATED */
+      g_assert(sanitized_message.str == buf);
+      log_msg_set_value(self, LM_V_MESSAGE, sanitized_message.str, sanitized_message.len);
+      self->flags |= LF_UTF8;
+    }
+  else
+    {
+      log_msg_set_value(self, LM_V_MESSAGE, (gchar *) src, left);
+
+      /* we don't need revalidation if sanitize already said it was valid utf8 */
+      if ((parse_options->flags & LP_VALIDATE_UTF8) &&
+          ((parse_options->flags & LP_SANITIZE_UTF8) == 0) &&
+          g_utf8_validate((gchar *) src, left, NULL))
+        self->flags |= LF_UTF8;
+    }
 
   return TRUE;
 }
